--
-- VHDL Architecture Cosmix_lib.SPI_CTR.Mk2
--
-- Created:
--          by - goodrick.UNKNOWN (PCCO)
--          at - 09:46:47 18/10/2005
--
-- using Mentor Graphics HDL Designer(TM) 2003.2 (Build 28)
--
LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_arith.all;

ENTITY SPI_CTR IS
   GENERIC( 
      ncoinc : integer := 6
   );
   PORT( 
      Rst   : IN     std_logic;
      SPCK  : IN     std_logic;
      SPDI  : IN     std_logic;
      Outz  : OUT    std_logic_vector (15 DOWNTO 0);
      SPCSn : IN     std_logic;
      CTEN  : OUT    std_logic;
      CTOE  : OUT    std_logic_vector (ncoinc-1 DOWNTO 0);
      CTCLR : OUT    std_logic;
      GCK   : IN     std_logic;
      SVar  : OUT    std_logic_vector (3 DOWNTO 0);
      SPDO  : OUT    std_logic;
      CTIN  : IN     std_logic_vector (31 DOWNTO 0)
   );

-- Declarations

END SPI_CTR ;

--
ARCHITECTURE Mk2 OF SPI_CTR IS
 --** Synchronous version **
   type StateType is (StandBy,GetArg,GetNReads,GetCWord,SendFrames,GetQual,DoQual,NotMe);
   signal MCState : StateType;
   signal SerialReg: std_logic_vector (31 downto 0) ;
   signal BitCount: integer range 0 to 31 ;
 --** Counter for use in State M/C
   signal SPCKL,SPCKLL,NextDOut,NextSRIn : std_logic ;
   signal OpCode,Arg,NReads,Qual         : integer range 0 to 255 ;
  
BEGIN
   
   --CTCLR <= '0'    ;
   --CTEN  <= '1'    ;
   Outz  <= SerialReg(15 downto 0) ;
--    
   StateMC: process (GCK,Rst,SPCSn,SPCK,SPCKL,MCState,SPCKLL)
   begin
      if((Rst = '1') or (SPCSn = '1') ) then --**RESET**
         SPCKLL         <= SPCK             ;
         SPCKL          <= SPCK             ;
         MCState        <= StandBy          ;
         SVAR           <= (others => '0')  ;
         SerialReg      <= (others => '0')  ;
         BitCount       <= 0                ;
         NReads         <= 0                ;
         NextDOut       <= '0'              ;
         NextSRIn       <= SPDI             ;
         SPDO           <= 'Z'              ;
         CTCLR <= '0';
         Opcode <= 0;
         Arg <= 0;
--  SPDO <= not SPCK ; -- DEBUG**
      elsif( rising_edge(GCK)) then
         SPCKLL         <= SPCKL            ;
         SPCKL          <= SPCK             ;

--**
         case MCState is
--
         when StandBy =>      --** Gets OpCode **
            SVar      <= "1010"                           ; -- DeBug
            CTCLR     <= '0'                              ;
            SPDO      <= 'Z'                              ;
            if((SPCKLL='0') and (SPCKL='1')) then  --**  LO-to-HI
               BitCount  <= BitCount      ;
               NextSRIn  <= SPDI          ; -- remember SPDI 'til falling SPCK
            elsif((SPCKLL='1') and (SPCKL='0')) then  --** HI-to-LO
               SerialReg <= SerialReg(30 downto 0) & NextSRIn ; --** shift in next bit
               if(BitCount >= 7) then
                  BitCount <= 0                                 ;
                  Opcode   <= conv_integer(unsigned(SerialReg(6 DOWNTO 0) & NextSRIn)) ;
                  if ((OpCode<8)or((OpCode>=16#80#)and(OpCode<16#88#))) then
                     MCState <= GetArg                    ;
                  else
                     MCState <= Notme                     ;
                  end if ;
               else
                  BitCount <= BitCount + 1                ;
               end if ;
            else
               BitCount <= BitCount                    ;                  
            end if ;
--            
         when GetArg =>
            SVar      <= "0001"                           ; -- DeBug
            SPDO      <= 'Z'                              ;
            if((SPCKLL='0') and (SPCKL='1')) then  --**  LO-to-HI
               BitCount  <= BitCount      ;
               NextSRIn  <= SPDI          ; -- remember SPDI 'til falling SPCK

            elsif((SPCKLL='1') and (SPCKL='0')) then  --** HI-to-LO
               SerialReg <= SerialReg(30 downto 0) & NextSRIn ; --** shift in next bit
               if(BitCount >= 7) then
                  BitCount <= 0                                 ;
                  Arg      <= conv_integer(unsigned(SerialReg(6 DOWNTO 0) & NextSRIn)) ;
                  if(OpCode<8) then -- must be a Counter Read
                     MCState  <= GetNReads                ;
                  else
                     MCState  <= GetQual                  ;
                  end if                                  ;
               else
                  BitCount <= BitCount + 1                ;
               end if ;
            end if ;
--            
         when GetNReads =>
            SVar       <= "0010"                          ; -- DeBug
            SPDO       <= 'Z'                             ;
            if((SPCKLL='0') and (SPCKL='1')) then  --**  LO-to-HI
               BitCount  <= BitCount      ;
               NextSRIn  <= SPDI          ; -- remember SPDI 'til falling SPCK

            elsif((SPCKLL='1') and (SPCKL='0')) then  --** HI-to-LO
               SerialReg <= SerialReg(30 downto 0) & NextSRIn ; --** shift in next bit
               if(BitCount >= 7) then
                  BitCount <= 0                                 ;
                  NReads   <= conv_integer(unsigned(SerialReg(6 DOWNTO 0) & NextSRIn)) ;
                  MCState  <= GetCWord                          ;
               else
                  BitCount <= BitCount + 1                ;
               end if ;
            end if ;
--            
         when GetCWord =>   --** Get Count Word
            SVar      <= "0011"                           ; -- DeBug
--            NextDOut  <= SerialReg(31)                    ;
            SPDO      <= SerialReg(31)                    ;
            SerialReg <= CTIN                             ; --** load count word
            BitCount  <= 0                                ;
            MCState   <= SendFrames                       ;
--            
         when SendFrames =>
            SVar      <= "0100"                           ; -- DeBug
            if((SPCKLL='0') and (SPCKL='1')) then  --**  LO-to-HI
               BitCount  <= BitCount + 1                  ;
               NextDOut  <= SerialReg(30)                 ;
               SerialReg <= SerialReg(30 downto 0) & SPDI ; --** shift in next bit
            elsif((SPCKLL='1') and (SPCKL='0')) then  --** HI-to-LO
               SPDO        <= NextDOut                    ;
               if(BitCount >=8) then
                  BitCount <= 0                           ;
                  NReads   <= NReads -1                   ;
                  if(NReads < 2) then
                     MCState <= StandBy                   ;
                  end if ;
               end if ;                  
            end if ;
--            
         when GetQual =>
            SVar      <= "1001"                           ; -- DeBug
            SPDO      <= 'Z'                              ;
            if((SPCKLL='0') and (SPCKL='1')) then  --**  LO-to-HI
               BitCount  <= BitCount      ;
               NextSRIn  <= SPDI          ; -- remember SPDI 'til falling SPCK

            elsif((SPCKLL='1') and (SPCKL='0')) then  --** HI-to-LO
               SerialReg <= SerialReg(30 downto 0) & NextSRIn ; --** shift in next bit
               if(BitCount >= 7) then
                  BitCount <= 0                                 ;
                  Qual     <= conv_integer(unsigned(SerialReg(6 DOWNTO 0) & NextSRIn)) ;
                  MCState  <= DoQual                      ;
               else
                  BitCount <= BitCount + 1                ;
               end if ;
            end if ;
--
         when DoQual =>
            SVar      <= "1010"                           ; -- DeBug
            SPDO      <= 'Z'                              ;
            if(OpCode = 16#81#)   then    -- Enable/Disable Ctrs
               if(Arg = 0)  then  -- Disable
                  CTEN   <= '0'  ;  -- sometime may want vector
               else
                  CTEN   <= '1'  ;  -- sometime may want vector
               end if ;
            elsif(OpCode = 16#82#) then   -- Clear Counters
               CTCLR  <= '1'                              ; -- Reset in StandBy
            end if                                        ;
            MCState <= StandBy                            ;
--
         when NotMe =>
            SVar      <= "1111"                           ; -- DeBug
            SPDO    <= 'Z' ;
--            
         end case ;
      end if ;      
   end process StateMC ;
--    
Decode: process(Arg, MCState)
 
   begin
      if((MCState=GetNReads) or (MCState=GetCWord) or (MCState=SendFrames)) then
         if   (Arg = 0) then CTOE <= "000001" ;
         elsif(Arg = 1) then CTOE <= "000010" ;
         elsif(Arg = 2) then CTOE <= "000100" ;
         elsif(Arg = 3) then CTOE <= "001000" ;
         elsif(Arg = 4) then CTOE <= "010000" ;
         elsif(Arg = 5) then CTOE <= "100000" ;
         else                CTOE <= "000000" ;
         end if;
      else
         CTOE <= "000000" ;
      end if;   
   end process Decode;
 --  
END Mk2;

