-------------------------------------------------------------------------------
-- single shot of 1 clock pulse upon leading edge of signal
-------------------------------------------------------------------------------


LIBRARY IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE IEEE.std_logic_misc.all;
USE IEEE.std_logic_unsigned.all;

entity singleshot is
  
  port (
    reset   : in  std_logic;
    clk    : in  std_logic;
    sigin  : in  std_logic;
    sigout : out std_logic);

end singleshot;

architecture singleshot_arch of singleshot is

  type statetype is (idle, doshot);
  signal currentstate : statetype;
  signal nextstate : statetype;

  signal siginint : std_logic;
  signal sigoutint : std_logic;
  
begin  -- singleshot_arch

sigout <= not(sigoutint or siginint);
  
  -- synchronise sigin input signal to avoid undefined states and/or race conditions
  syncsigin: process (sigoutint, sigin)
  begin  -- process syncsigin
    if sigoutint = '1' then
      siginint <= '0';
    elsif sigin'event and sigin = '1' then
      siginint <= '1';
    end if;
  end process syncsigin;
  
  clkstates: process (reset, clk)
  begin  -- process clkstates
    if reset = '1' then
      currentstate <= idle;
    elsif clk'event and clk = '1' then
      currentstate <= nextstate;
    end if;
  end process clkstates;
  
  singleshotstates: process (currentstate, siginint)
  begin  -- process singleshotstates
    case currentstate is
      when idle =>              sigoutint <= '0';
                                if siginint = '1' then
                                  nextstate <= doshot;
                                  else
                                    nextstate <= idle;
                                end if;
      when doshot =>            sigoutint <= '1';
                                nextstate <= idle;
      when others =>            sigoutint <= '0';
                                nextstate <= idle;
    end case;
  end process singleshotstates;
  

end singleshot_arch;
