library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity DataSynchroniser is
    Port ( serialIn : in  STD_LOGIC;
           dataOut : out  STD_LOGIC_VECTOR (7 downto 0);
			  newData : out STD_LOGIC;
           clk4x : in  STD_LOGIC;
           rst : in  STD_LOGIC);
end DataSynchroniser;

architecture Behavioral of DataSynchroniser is
	type StateType is (
		idle,
		pause,
		readData,
		done
	);
	
	signal state, stateAfter : StateType;
	signal clkState : unsigned (1 downto 0);
	signal dataState : unsigned (2 downto 0);
	
begin
	stateMachine : process (clk4x, rst)
	begin
		if (clk4x'event and clk4x = '1') then
			if (rst = '1') then
				state <= idle;
				stateAfter <= idle;
				newData <= '0';
				clkState <= (others => '0');
				dataState <= (others => '0');
			else
				case state is
					when idle =>
						if (serialIn = '0') then
							state <= readData;
							clkState <= to_unsigned(1,2);
						end if;
						
					when pause =>
						state <= stateAfter;
						
					when readData =>
						if (clkState = 0) then
							dataOut(to_integer(dataState)) <= serialIn;
							if (dataState = 7) then
								state <= done;
							else
								dataState <= dataState + 1;
							end if;
						end if;
					when done =>
						state <= done;
				end case;
			end if;
		end if;
	end process stateMachine;
end Behavioral;

