----------------------------------------------------------------------------------
-- CROCS - Flasher
-- Divides the clock input, useful for flashing an LED.
-- Flshing frequency is CLK_FREQ/2^(Divisor+1).
--
-- Fergus Noble 2011
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity Flasher is
	 Generic ( Divisor : integer := 23 ); -- Divides clock by 2^(Divisor+1)
    Port ( clk : in  STD_LOGIC;
           reset : in  STD_LOGIC;
			  flash : out  STD_LOGIC
			);
end Flasher;

architecture Behavioral of Flasher is
	signal count : std_logic_vector(Divisor downto 0);
begin
	flash <= count(Divisor);
	
	process (clk, reset)
	begin
	
		if (reset = '1') then
			count <= (others=>'0');
		elsif (clk'event and clk = '1') then
			count <= count + 1;
		end if;
		
	end process;
	
end Behavioral;

