----------------------------------------------------------------------------------
-- CROCS - PulseStretcher
-- Stretches an input pulse so that it is 2^Divisor/CLK_FREQ long.
--
-- Fergus Noble 2011
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity PulseStretcher is
	 Generic ( Divisor : integer := 23 ); -- Divides clock by 2^Divisor
    Port ( pulseIn : in  STD_LOGIC;
           clk : in  STD_LOGIC;
           rst : in  STD_LOGIC;
           pulseOut : out  STD_LOGIC);
end PulseStretcher;

architecture Behavioral of PulseStretcher is
	signal count : std_logic_vector(Divisor downto 0);
	signal counting : std_logic;
begin
	pulseOut <= counting;
	
	process (clk, rst, pulseIn)
	begin
		if (rst = '1') then
			count <= (others=>'0');
			counting <= '0';
		elsif (clk'event and clk = '1') then
			if (counting = '1') then 
				count <= count + 1;
				if (count(Divisor) = '1') then
					counting <= '0';
				end if;
			elsif (pulseIn = '1') then
				counting <= '1';
				count <= (others=>'0');
			end if;
		end if;
	end process;

end Behavioral;