/*
 * ==================================================
 * Cosmixer
 *     Started 21-Oct-2005
 *     Based on CosmixPick and mJUSB1
 *     Reworked by SW to merge best bits from Wotton/Goodrick/Ashford
 *     SW 2012, new incarnation as Cosmixer
 * ==================================================
 */
package mjg.instrument;
import java.util.Date;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.applet.Applet;
import java.applet.AudioClip;
import java.net.URL;
import java.io.*;
import java.util.prefs.Preferences;
import mjg.instrument.* ;

public class Cosmixer extends javax.swing.JFrame implements ActionListener
{
    Preferences rootPrefs;
    Preferences prefs; 

    private Timer  timer;
    int ticks;
    int sleepyTime = 200;
    private static JniCosmix usb;
    private static boolean isUsbOpen = false;
    private static boolean isMuted = false;
    static final int nCoincidences = 6;
    static final int nStreams = 4;

    // OpCodes: OR with OpWriteExec for Write or Execute
    static final int OpCtrVal        = 0x00  ;
    static final int OpCtrEnable     = 0x01  ;
    static final int OpCtrClear      = 0x02  ;
    static final int OpCtrTime       = 0x03  ;
    static final int OpMCUStatus     = 0x10  ;
    static final int OpMDAC          = 0x20  ;
    static final int OpMaskReq       = 0x08  ;
    static final int OpMaskVeto      = 0x09  ;
    static final int OpStreamStretch = 0x0a  ;
    static final int OpStreamDelay   = 0x0c  ;
    static final int OpWriteExec     = 0x80  ;

    short [] adc;

// Just a specialisation of the ADC0 and ADC1 displayed as HV Vmon and Imon

public class HvPanel extends JPanel implements ActionListener
{
  JTextField vmonTF;
  JTextField imonTF;

  public HvPanel()
  {
    setLayout(new java.awt.GridLayout(2,2));
    setBorder(BorderFactory.createTitledBorder("HV monitor"));

    add(new JLabel("Voltage"));
    vmonTF = new JTextField("0.0");
    vmonTF.setEditable(false);
    vmonTF.setColumns(8);
    vmonTF.setBackground(java.awt.Color.darkGray);
    vmonTF.setForeground(java.awt.Color.orange);
    vmonTF.setFont(new java.awt.Font("Monospaced", 1, 32));
    vmonTF.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
    vmonTF.addActionListener(this);
    add(vmonTF);

    add(new JLabel("Current"));
    imonTF = new JTextField("0.0");
    imonTF.setEditable(false);
    imonTF.setColumns(8);
    imonTF.setBackground(java.awt.Color.darkGray);
    imonTF.setForeground(java.awt.Color.orange);
    imonTF.setFont(new java.awt.Font("Monospaced", 1, 32));
    imonTF.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
    imonTF.addActionListener(this);
    add(imonTF);
  }

  public void actionPerformed( ActionEvent e )
  {
    String cmd = e.getActionCommand();
    if ( cmd == null )
    {
    }
  }

  public void setValue(short v, short i)
  {
    vmonTF.setText(String.format("%1$g",adcCalibrationPanel.calibrate(0,v)));
    imonTF.setText(String.format("%1$g",adcCalibrationPanel.calibrate(1,i)));
    return;
  }

}

public class CoincidenceSetupPanel extends JPanel implements ActionListener
{
  int index;
  int mask;
  JCheckBox [] streamCB;

  public CoincidenceSetupPanel( int id )
  {
    index = id;
    streamCB = new JCheckBox[4];

    setLayout(new java.awt.GridLayout(1,4));
    setBorder(BorderFactory.createTitledBorder(String.format("Coincidence %d",index)));

    for ( int i=0; i<Cosmixer.nStreams; i++ )
    {
      streamCB[i] = new JCheckBox(String.format("%d",i),false);
      streamCB[i].addActionListener( this );
      streamCB[i].setToolTipText( "Select to include stream in coincidence" );
      add(streamCB[i]);
    }
  }

  public void sendCoincidenceMask()
  {
    sendWriteOp(OpMaskReq,index,mask);
  }

  public void actionPerformed( ActionEvent e )
  {
    for (int i=0; i<Cosmixer.nStreams; i++)
    {
       mask = streamCB[i].isSelected() ? (mask | (1<<i)) : (mask & ~(1<<i));
    }
    prefs.putInt(String.format("coincidence_mask%1$d",index),mask);

    sendCoincidenceMask();

  }

  public void initialiseFromPreferences()
  {
    mask = prefs.getInt(String.format("coincidence_mask%1$d",index),0x3);
    for ( int i=0; i<Cosmixer.nStreams; i++ )
    {
      streamCB[i].setSelected((mask&1<<i)!=0);
    }
    sendCoincidenceMask();
  }

}

public class AdcCalibrationPanel extends JPanel implements ActionListener
{
  JTextField [] scaleTF;
  JTextField [] offsetTF;

  public AdcCalibrationPanel()
  {
    scaleTF = new JTextField[4];
    offsetTF = new JTextField[4];
    setLayout(new java.awt.GridLayout(2,5));
    setBorder(BorderFactory.createTitledBorder("ADC[0..3]"));
    add(new JLabel("Scale factor"));
    for (int i=0;i<4;i++ )
    {
      scaleTF[i] = new JTextField("1.0");
      scaleTF[i].setEditable(true);
      scaleTF[i].setColumns(6);
      scaleTF[i].setFont(new java.awt.Font("Monospaced", 1, 16));
      scaleTF[i].setHorizontalAlignment(javax.swing.JTextField.RIGHT);
      scaleTF[i].addActionListener(this);
      scaleTF[i].setActionCommand("scale");
      add(scaleTF[i]);
    }
    add(new JLabel("Offset"));
    for (int i=0;i<4;i++ )
    {
      offsetTF[i] = new JTextField("0.0");
      offsetTF[i].setEditable(true);
      offsetTF[i].setColumns(6);
      offsetTF[i].setFont(new java.awt.Font("Monospaced", 1, 16));
      offsetTF[i].setHorizontalAlignment(javax.swing.JTextField.RIGHT);
      offsetTF[i].addActionListener(this);
      offsetTF[i].setActionCommand("offset");
      add(offsetTF[i]);
    }
  }

  public void actionPerformed( ActionEvent e )
  {
    String cmd = e.getActionCommand();
    if ( cmd == null )
    {
    }
    else if ( cmd.equals( "scale" ) )
    {
        String sval = ((JTextField)e.getSource()).getText();
	if ( e.getSource().equals(scaleTF[0]) )
	{
	  prefs.put("adc_scale0",sval);
        }
        else if ( e.getSource().equals(scaleTF[1]) )
	{
	  prefs.put("adc_scale1",sval);
        }
        else if ( e.getSource().equals(scaleTF[2]) )
	{
	  prefs.put("adc_scale2",sval);
        }
        else if ( e.getSource().equals(scaleTF[3]) )
	{
	  prefs.put("adc_scale3",sval);
        }
    }
    else if ( cmd.equals( "offset" ) )
    {
        String sval = ((JTextField)e.getSource()).getText();
	if ( e.getSource().equals(offsetTF[0]) )
	{
	  prefs.put("adc_offset0",sval);
        }
        else if ( e.getSource().equals(offsetTF[1]) )
	{
	  prefs.put("adc_offset1",sval);
        }
        else if ( e.getSource().equals(offsetTF[2]) )
	{
	  prefs.put("adc_offset2",sval);
        }
        else if ( e.getSource().equals(offsetTF[3]) )
	{
	  prefs.put("adc_offset3",sval);
        }
    }
  }

  public void initialiseFromPreferences()
  {
    for (int i=0;i<4;i++ )
    {
      scaleTF[i].setText(prefs.get(String.format("adc_scale%1$d",i),"1.0"));
      offsetTF[i].setText(prefs.get(String.format("adc_offset%1$d",i),"0.0"));
    }
  }

  public double calibrate( int i, short adc )
  {
    double scale = Double.parseDouble(scaleTF[i].getText());
    double offset = Double.parseDouble(offsetTF[i].getText());
    return (adc + offset)*scale;
  }

}

public class AdcPanel extends JPanel implements ActionListener
{
    JTextField [] adcTF;

    public AdcPanel()
    {

        adcTF = new JTextField[4];
        setLayout(new java.awt.GridLayout(1,5));
        setBorder(BorderFactory.createTitledBorder("ADC[0..3]"));

        for (int i=0;i<4;i++ )
	{
	  adcTF[i] = new JTextField();
          adcTF[i].setEditable(false);
          adcTF[i].setColumns(4);
	  adcTF[i].setFont(new java.awt.Font("Monospaced", 1, 16));
          adcTF[i].setHorizontalAlignment(javax.swing.JTextField.RIGHT);
          add(adcTF[i]);
        }

        JButton button = new JButton("Calibrate...");
        button.addActionListener(this);
        button.setActionCommand("calibrate");
        add(button);

    }

  public void actionPerformed( ActionEvent e )
  {
    String cmd = e.getActionCommand();
    if ( cmd == null )
    {
    }
    else if ( cmd.equals( "calibrate" ) )
    {
      adcCalibrationFrame.setVisible(true);
    }
  }

  public void setValue(int i, short adc)
  {
    adcTF[i].setText(String.format("%1$g",adcCalibrationPanel.calibrate(i,adc)));
    return;
  }

}

public class ThresholdPanel extends JPanel implements ChangeListener, ActionListener
{
    JSlider bar;
    JTextField text;
    JTextField stretchTF;
    JTextField delayTF;
    int index;
    int threshold;
    int stretch;
    int delay;

    public ThresholdPanel( int idx )
    {
        index = idx;

        setLayout(new java.awt.GridBagLayout());
        setBorder(BorderFactory.createTitledBorder(String.format("Stream %1$d",idx)));

        GridBagConstraints gbC = new GridBagConstraints();

        gbC.fill = GridBagConstraints.BOTH;
        gbC.anchor = GridBagConstraints.CENTER;
        gbC.insets = new Insets(1,1,1,1);
        gbC.weightx = 1.0;

        bar = new JSlider();
        bar.setMaximum(255);
        bar.setMajorTickSpacing(32);
        bar.createStandardLabels(32);
        bar.setPaintLabels(true);
        bar.setPaintTicks(true);
        bar.addChangeListener(this);
        bar.setBorder(BorderFactory.createTitledBorder("Threshold"));

        add(bar,gbC);

        text = new JTextField();
        text.setEditable(false);
	text.setFont(new java.awt.Font("Monospaced", 1, 28));
        text.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        text.setBorder(BorderFactory.createTitledBorder("Vth [mV]"));

        add(text,gbC);

        stretchTF = new JTextField();
        stretchTF.setEditable(true);
	stretchTF.setFont(new java.awt.Font("Monospaced", 1, 28));
        stretchTF.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        stretchTF.addActionListener(this);
        stretchTF.setActionCommand("stretch");
        stretchTF.setBorder(BorderFactory.createTitledBorder("Nstr [20ns]"));
	
        add(stretchTF,gbC);

	delayTF = new JTextField();
        delayTF.setEditable(true);
	delayTF.setFont(new java.awt.Font("Monospaced", 1, 28));
        delayTF.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        delayTF.addActionListener(this);
        delayTF.setActionCommand("delay");
        delayTF.setBorder(BorderFactory.createTitledBorder("Delay"))

	gbC.gridwidth = GridBagConstraints.REMAINDER;
	add(delayTF,gbC);

        gbC.gridwidth = 1;

    }

    public void setValue( int th )
    {
	threshold = th;
        prefs.putInt(String.format("threshold%1$d",index),threshold);
        bar.setValue(threshold);
        Double dth = 250*threshold/256.0;
        text.setText(String.format("%1$5.1f",dth));
    }

    public void setStretch( int str )
    {
	stretch = str;
        prefs.putInt(String.format("stretch%1$d",index),stretch);
        stretchTF.setText(String.format("%1$d",stretch));
    }

    public void setDelay( int str )
    {
	delay = str;
        prefs.putInt(String.format("delay%1$d",index),delay);
        delayTF.setText(String.format("%1$d",delay));
    }

    public void sendTHold()
    {
	// Encode DAC command. Byte-swapped.
        int DAC = ((threshold & 0xf)<<12) | ((8-index)<<4) | (threshold>>4);

        sendWriteOp(OpMDAC,0,DAC);

	return;
    }

    public void sendStretch()
    {
      sendWriteOp(OpStreamStretch,index,stretch);
    }

    public void sendDelay()
    {
	sendWriteOp(OpStreamDelay,index,delay);
    }


    public void stateChanged(javax.swing.event.ChangeEvent evt)
    {
        threshold = bar.getValue();
        prefs.putInt(String.format("threshold%1$d",index),threshold);
        Double dth = 250*threshold/256.0;
        text.setText(String.format("%1$5.1f",dth));
	sendTHold() ;
    }

    public void actionPerformed(ActionEvent e)
    {
      String cmd = e.getActionCommand();
      if ( cmd.equals( "stretch" ) )
      {
        String sval = ((JTextField)e.getSource()).getText();
        stretch = Integer.parseInt(sval);
        prefs.putInt(String.format("stretch%1$d",index),stretch);
        sendStretch();
      }
      else if ( cmd.equals( "delay" ) )
      {
        String sval = ((JTextField)e.getSource()).getText();
        delay = Integer.parseInt(sval);
        prefs.putInt(String.format("delay%1$d",index),delay);
        sendDelay();
      }
    }
}

public class CounterPanel extends JPanel implements ActionListener
{
    int id;
    int count = 0;
    int lastCount = 0;
    double rate = 0.0;
    int lastRateCount = 0;
    long lastRateTime = 0;
    JTextField countTF;
    JTextField rateTF;
    boolean playSoundOnChange;

    public CounterPanel( int ctr, String bT, int fS, Color fC, boolean sound )
    {
      id = ctr;
      playSoundOnChange = sound;

      setLayout(new java.awt.GridLayout(2, 1));
      setBorder( BorderFactory.createTitledBorder(bT));

      countTF = new JTextField();
      countTF.setColumns(10);
      countTF.setBackground(java.awt.Color.darkGray);
      countTF.setFont(new java.awt.Font("Monospaced", 1, fS));
      countTF.setForeground(fC);
      countTF.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
      add(countTF);

      rateTF = new JTextField();
      rateTF.setColumns(10);
      rateTF.setBackground(java.awt.Color.darkGray);
      rateTF.setFont(new java.awt.Font("Monospaced", 1, fS));
      rateTF.setForeground(fC);
      rateTF.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
      add(rateTF);

    }

    public void actionPerformed( ActionEvent e )
    {
    }

    public void updateCount()
    {
	int [] outPacket = new int[68];
	int [] inPacket = new int[68];

        outPacket[0] = OpCtrVal;
        outPacket[1] = id;
        outPacket[2] = 4;
        outPacket[3] = 0;
        outPacket[4] = 4; // NReads

        try
	{
	  sendUSB(outPacket);
          getUSB(inPacket);
	}
	catch ( IOException ioe )
	{
	  usb.mjgCloseUSB();
	  isUsbOpen = false;
	  deviceControlPanel.deviceSerialCombo.setBackground(Color.red);
	}

        count = inPacket[4]<<24 | inPacket[5]<<16 | inPacket[6]<<8 | inPacket[7];

        if ( playSoundOnChange && !isMuted && (count != lastCount) ) audioClip.play();

        lastCount = count;

        countTF.setText(String.format("%1$010d",count)) ;
    }

    public void updateRate( Date tod )
    {
      long now = tod.getTime();
      rate = ((now - lastRateTime) == 0 ) ? 0.0 : 1000.0*(double)(count-lastRateCount)/(double)(now-lastRateTime);
      rateTF.setText(String.format("%1$7.2g Hz",rate)) ;
      lastRateCount = count;
      lastRateTime = now;
    }

    public int getCount() { return count; }

    public double getRate() { return rate; }

    public void clear()
    {
      count = 0;
      lastCount = 0;
    }

    public void writeData( FileWriter stream )
    {
      try
      {
        stream.write(String.format("%1$d , ",count));
      }
      catch ( IOException ioe )
      {
        System.out.println("Logging exception");
      }
      
    }

}
    
public class DeviceControlPanel extends JPanel implements ActionListener
{
  JButton connectButton;
  JComboBox deviceSerialCombo;
  int deviceSerial;

  public DeviceControlPanel()
  {
    deviceSerial = rootPrefs.getInt("Cosmixer.deviceSerial", 1);
    prefs = Preferences.userNodeForPackage(Cosmixer.class).node("Cosmixer/" + Integer.toString(deviceSerial));

    setLayout( new GridBagLayout() );
    GridBagConstraints gbC = new GridBagConstraints();

    gbC.fill = GridBagConstraints.NONE;
    gbC.anchor = GridBagConstraints.CENTER;

    connectButton = new JButton( "Connect" );
    connectButton.setActionCommand("connect");
    connectButton.addActionListener( this );

    gbC.fill = GridBagConstraints.VERTICAL;
    gbC.anchor = GridBagConstraints.CENTER;
    add( connectButton, gbC );

    deviceSerialCombo = new JComboBox();
    deviceSerialCombo.setEditable(true);

    int deviceList[] = new int[6];
    int nfound = usb.enumerateUSB("MJG Cosmic",deviceList);
    System.out.println( "Found " + nfound + " devices" );

    deviceSerialCombo.setMaximumRowCount(6);

    boolean foundPreferredDevice = false;
    int idx = 0;
    for ( idx=0;idx<nfound;idx++ )
    {
      deviceSerialCombo.addItem( Integer.toString(deviceList[idx]) );
      if ( deviceList[idx] == deviceSerial )
      {
	deviceSerialCombo.setSelectedIndex(idx);
        foundPreferredDevice = true;
      }
    }

    if (!foundPreferredDevice)
    {
      deviceSerialCombo.addItem( Integer.toString(deviceSerial) );
      deviceSerialCombo.setSelectedIndex(idx);
    }

    deviceSerialCombo.setBackground(Color.red);
    deviceSerialCombo.setActionCommand("serial");
    deviceSerialCombo.addActionListener( this );
    deviceSerialCombo.setToolTipText( "Device connect serial number" );

    gbC.fill = GridBagConstraints.VERTICAL;
    gbC.anchor = GridBagConstraints.CENTER;
    gbC.gridwidth = GridBagConstraints.REMAINDER;
    add( deviceSerialCombo, gbC );

    return;
  }

  public void actionPerformed( ActionEvent e )
  {
    String cmd = e.getActionCommand();
    if ( cmd.equals( "connect" ) )
    {
      if ( isUsbOpen == false )
      {
        int rc = usb.mjgOpenUSB( "MJG Cosmic", deviceSerial );
        System.out.println( "mjgOpenUSB() rc = " + rc + " Serial # = " + deviceSerial );

        if ( rc == 0 )
	{
          isUsbOpen = true;
	}

      }
      else
      {
	isUsbOpen = false;
        int rc = usb.mjgCloseUSB();
        System.out.println( "mjgCloseUsb() rc = " + rc );

        rc = usb.mjgOpenUSB( "MJG Cosmic", deviceSerial );
        System.out.println( "mjgOpenUSB() rc = " + rc + " Serial # = " + deviceSerial );

        isUsbOpen = (rc == 0);
      }

      if ( isUsbOpen )
      {
        prefs = Preferences.userNodeForPackage(Cosmixer.class).node("Cosmixer/" + Integer.toString(deviceSerial));
	deviceControlPanel.deviceSerialCombo.setBackground(Color.green);

	// Disable and clear counters
        sendWriteOp(OpCtrEnable,0,0x3f)  ; // Disable All
        sendWriteOp(OpCtrClear,0,0x3f)  ; // Clear All

	// Initialise the coincidence patterns
        for (int i=0;i<Cosmixer.nCoincidences;i++)
	{
	  coPanels[i].initialiseFromPreferences();
	}

	// Set the thresholds
        int th;
        int str;
	int del;
        for ( int ith=0;ith<4;ith++)
	{
          th = prefs.getInt( String.format("threshold%1$d",ith), 50 );
          thresholdPanel[ith].setValue(th);
          thresholdPanel[ith].sendTHold();
          str = prefs.getInt( String.format("stretch%1$d",ith), 2 );
          thresholdPanel[ith].setStretch(str);
          thresholdPanel[ith].sendStretch();
	  del = prefs.getInt( String.format("delay%1$d",ith), 0 );
          thresholdPanel[ith].setDelay(del);
          thresholdPanel[ith].sendDelay();
	}

	// Initialise the ADC calibration
        adcCalibrationPanel.initialiseFromPreferences();
      }
      else
      {
	deviceControlPanel.deviceSerialCombo.setBackground(Color.red);
      }
    }
    else if ( cmd.equals( "serial" ) )
    {
      Integer deviceSerialInteger = Integer.valueOf((String)deviceSerialCombo.getSelectedItem());
      deviceSerial = deviceSerialInteger.intValue();
      rootPrefs.putInt( "Cosmixer.deviceSerial", deviceSerial );
      System.out.println( "deviceSerial = " + deviceSerial );
    }
    return;
  }
}


// Creates new top level form

    public Cosmixer() {

        rootPrefs = Preferences.userNodeForPackage(Cosmixer.class).node("Cosmixer");
        prefs = Preferences.userNodeForPackage(Cosmixer.class).node("Cosmixer/" + Integer.toString(0));

        Container cP = getContentPane();
        cP.setLayout(new BorderLayout());

        JMenuBar mB = new JMenuBar();
        setJMenuBar( mB );

        adc = new short[4];

        initComponents();

        timer = new Timer(sleepyTime, this );
        timer.setInitialDelay(0);
        timer.setCoalesce( false );
        
        timer.start();
    } 
   
    public void updateStatus()
    {
	int [] outPacket = new int[68];
	int [] inPacket = new int[68];

        outPacket[0] = OpMCUStatus;
        outPacket[1] = 0;
        outPacket[2] = 4;
        outPacket[3] = 0;
        outPacket[4] = 12; // NReads

        try
	{
	  sendUSB(outPacket);
          getUSB(inPacket);
	}
	catch ( IOException ioe )
	{
	  usb.mjgCloseUSB();
	  isUsbOpen = false;
	  deviceControlPanel.deviceSerialCombo.setBackground(Color.red);
	}

        adc[0] = (short)(((inPacket[9]&0xff) << 8) | (inPacket[8]&0xff));
        adc[1] = (short)(((inPacket[11]&0xff) << 8) | (inPacket[10]&0xff));
        adc[2] = (short)(((inPacket[13]&0xff) << 8) | (inPacket[12]&0xff));
        adc[3] = (short)(((inPacket[15]&0xff) << 8) | (inPacket[14]&0xff));
    }

     public void actionPerformed( ActionEvent e )
     {
        //-- This deals with the periodic tasks

        String cmd = e.getActionCommand();

        if ( cmd == null )
        {
          ticks++;
          Date tod = new Date(); 
 
 // Update count on every tick
          for (int CtrIdx=0;CtrIdx<Cosmixer.nCoincidences;CtrIdx++)
          {
	      counterPanel[CtrIdx].updateCount();
          }
          updateStatus();
          for ( int i=0;i<4;i++ )
	  {
	    adcPanel.setValue(i,adc[i]);
	  }
          hvPanel.setValue(adc[0],adc[1]);

// Update rate from time to time
          if ( (ticks%100) == 0 )
	  {
            for (int CtrIdx=0;CtrIdx<Cosmixer.nCoincidences;CtrIdx++)
	    {
              counterPanel[CtrIdx].updateRate( tod );
	    }
	  }

// Log data from time to time
          int loggingTicks = (1000/sleepyTime)*dataLoggerControl.getLoggingInterval();
          if ( (ticks%loggingTicks) == 0 && dataLoggerControl.isLogging() )
	  {
            dataLoggerControl.writeTimestamp( tod );

            for (int CtrIdx=0;CtrIdx<Cosmixer.nCoincidences;CtrIdx++)
	    {
              counterPanel[CtrIdx].writeData( dataLoggerControl.getStream() );
	    }
            try
	    {
              dataLoggerControl.getStream().write(String.format("%1$d , %2$d , %3$d , %4$d",adc[0],adc[1],adc[2],adc[3]));
	    }
            catch ( IOException ioe )
	    {
		System.out.println("Error logging pressure");
	    }
            dataLoggerControl.flush();
	  }
// Update chart every minute
          int chartTicks = (1000/sleepyTime)*60;
          if ( (ticks%chartTicks) == 0 )
	  {
	    double pressure = adcCalibrationPanel.calibrate(3,adc[3]);
	    dataLoggerControl.updateCharts( tod, counterPanel[2].getRate(), pressure );
	  }
       }

       return;
    }


// This method is called from within the constructor to initialize the form.

    private void initComponents()
    {
	URL resourceURL;
        try
	{
	  resourceURL = Cosmixer.class.getResource("muted24.gif");
          mutedIcon = new ImageIcon(resourceURL);
	  resourceURL = Cosmixer.class.getResource("unmuted24.gif");
          unmutedIcon = new ImageIcon(resourceURL);
	  audioURL = Cosmixer.class.getResource("ding.wav");
          audioClip = Applet.newAudioClip(audioURL);
	}
	catch ( Exception e )
        {
          System.out.println( "AudioClip exception" );
        }

        MainLabel = new javax.swing.JLabel();
        StartStopButton = new javax.swing.JToggleButton();
        LogData = new javax.swing.JButton();
        ClearButton = new javax.swing.JButton();

        thresholdPanel = new ThresholdPanel [4];
    
        thresholdPanel[0] = new ThresholdPanel(0);
        thresholdPanel[1] = new ThresholdPanel(1);
        thresholdPanel[2] = new ThresholdPanel(2);
        thresholdPanel[3] = new ThresholdPanel(3);

        counterPanel = new CounterPanel[Cosmixer.nCoincidences];

        counterPanel[0] = new CounterPanel(0, new String("Counter A"),32,Color.yellow, false);
        counterPanel[1] = new CounterPanel(1, new String("Counter B"),32,Color.yellow, false);
        counterPanel[2] = new CounterPanel(2, new String("Counter C"),32,Color.yellow, false);
        counterPanel[3] = new CounterPanel(3, new String("Counter D"),32,Color.yellow, false);
        counterPanel[4] = new CounterPanel(4, new String("Counter E"),32,Color.yellow, false);
        counterPanel[5] = new CounterPanel(5, new String("Counter F"),32,Color.yellow, false);

	logDataFrame = new JFrame( "Data logger" );
        dataLoggerControl = new DataLoggerControl();
        logDataFrame.getContentPane().add(dataLoggerControl);
        logDataFrame.setDefaultCloseOperation(WindowConstants.HIDE_ON_CLOSE);
        logDataFrame.pack();

        Container cP = getContentPane();

        JPanel mainPanel = new JPanel();

        GridBagLayout gB = new GridBagLayout(); 
        mainPanel.setLayout( gB );
        GridBagConstraints gbC = new GridBagConstraints();

        gbC.fill = GridBagConstraints.NONE;
        gbC.anchor = GridBagConstraints.CENTER;
        gbC.insets = new Insets(0,0,0,0);
        gbC.weightx = 1.0;
        gbC.weighty = 1.0;

        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        });

        MainLabel.setFont(new java.awt.Font("Comic Sans MS", 1, 36));
        MainLabel.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        MainLabel.setText("Cosmixer");

        gB.setConstraints(MainLabel,gbC);
        mainPanel.add(MainLabel);

        deviceControlPanel = new DeviceControlPanel();
        gbC.fill = GridBagConstraints.BOTH;
        gbC.anchor = GridBagConstraints.WEST;
        gbC.gridwidth = GridBagConstraints.REMAINDER;
        gB.setConstraints( deviceControlPanel, gbC );
        mainPanel.add( deviceControlPanel );
        gbC.gridwidth = 1;
        gbC.fill = GridBagConstraints.NONE;
        gbC.anchor = GridBagConstraints.CENTER;

// MCU ADC readings

        adcPanel = new AdcPanel();

        adcCalibrationPanel = new AdcCalibrationPanel();
	adcCalibrationFrame = new JFrame( "ADC calibration" );
        adcCalibrationFrame.getContentPane().add(adcCalibrationPanel);
        adcCalibrationFrame.setDefaultCloseOperation(WindowConstants.HIDE_ON_CLOSE);
        adcCalibrationFrame.pack();

        gbC.gridwidth = GridBagConstraints.REMAINDER;
        gbC.fill = GridBagConstraints.HORIZONTAL;
        gB.setConstraints( adcPanel, gbC );
        mainPanel.add(adcPanel);
        gbC.gridwidth = 1;
        gbC.fill = GridBagConstraints.NONE;

// HV interpretation of ADC[0,1]

        hvPanel = new HvPanel();
	hvFrame = new JFrame( "HV monitor" );
        hvFrame.getContentPane().add(hvPanel);
        hvFrame.setDefaultCloseOperation(WindowConstants.HIDE_ON_CLOSE);
        hvFrame.pack();
	//        hvFrame.setVisible(true);

// Threshold panels

        gbC.gridwidth = GridBagConstraints.REMAINDER;
        gbC.fill = GridBagConstraints.BOTH;
        gB.setConstraints( thresholdPanel[0], gbC );
        mainPanel.add(thresholdPanel[0]);
        gbC.gridwidth = 1;
        gbC.fill = GridBagConstraints.NONE;

        gbC.gridwidth = GridBagConstraints.REMAINDER;
        gbC.fill = GridBagConstraints.BOTH;
        gB.setConstraints( thresholdPanel[1], gbC );
        mainPanel.add(thresholdPanel[1]);
        gbC.gridwidth = 1;
        gbC.fill = GridBagConstraints.NONE;

        gbC.gridwidth = GridBagConstraints.REMAINDER;
        gbC.fill = GridBagConstraints.BOTH;
        gB.setConstraints( thresholdPanel[2], gbC );
        mainPanel.add(thresholdPanel[2]);
        gbC.gridwidth = 1;
        gbC.fill = GridBagConstraints.NONE;

        gbC.gridwidth = GridBagConstraints.REMAINDER;
        gbC.fill = GridBagConstraints.BOTH;
        gB.setConstraints( thresholdPanel[3], gbC );
        mainPanel.add(thresholdPanel[3]);
        gbC.gridwidth = 1;
        gbC.fill = GridBagConstraints.NONE;

        gB.setConstraints( counterPanel[0], gbC );
        mainPanel.add(counterPanel[0]);

        gbC.gridwidth = GridBagConstraints.REMAINDER;
        gB.setConstraints( counterPanel[1], gbC );
        mainPanel.add(counterPanel[1]);
        gbC.gridwidth = 1;

        gB.setConstraints( counterPanel[2], gbC );
        mainPanel.add(counterPanel[2]);

        gbC.gridwidth = GridBagConstraints.REMAINDER;
        gB.setConstraints( counterPanel[3], gbC );
        mainPanel.add(counterPanel[3]);
        gbC.gridwidth = 1;

        gB.setConstraints( counterPanel[4], gbC );
        mainPanel.add(counterPanel[4]);

        gbC.gridwidth = GridBagConstraints.REMAINDER;
        gB.setConstraints( counterPanel[5], gbC );
        mainPanel.add(counterPanel[5]);
        gbC.gridwidth = 1;

        JPanel buttonPanel = makeButtonPanel();

        gbC.gridwidth = GridBagConstraints.REMAINDER;
        gB.setConstraints( buttonPanel, gbC );
        mainPanel.add(buttonPanel);
        gbC.gridwidth = 1;

        cP.add( mainPanel, BorderLayout.NORTH );

// Panel to configure the coincicence patterns

        JFrame coFrame;
        coPanels = new CoincidenceSetupPanel[Cosmixer.nCoincidences];
	coFrame = new JFrame( "Coincidence pattern setup" );
        coFrame.setLayout(new java.awt.GridLayout(3,2));
        for (int i=0; i<Cosmixer.nCoincidences; i++ )
	{
	    coPanels[i] = new CoincidenceSetupPanel(i);
            coFrame.getContentPane().add(coPanels[i]);
	}
        coFrame.setDefaultCloseOperation(WindowConstants.HIDE_ON_CLOSE);
        coFrame.pack();
        coFrame.setVisible(true);

    }

private JPanel makeButtonPanel()
{
        JPanel buttonPanel = new JPanel();
        GridBagLayout gB = new GridBagLayout();
        GridBagConstraints gbC = new GridBagConstraints();

        buttonPanel.setLayout(gB);

        gbC.fill = GridBagConstraints.VERTICAL;
        gbC.anchor = GridBagConstraints.CENTER;
        gbC.insets = new Insets(0,0,0,0);
        gbC.weightx = 1.0;
        gbC.weighty = 1.0;

        StartStopButton.setFont(new java.awt.Font("Comic Sans MS", 1, 14));
        StartStopButton.setText("Start");
        StartStopButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                StartStopButtonMouseClicked(evt);
            }
        });

        gB.setConstraints(StartStopButton, gbC );
        buttonPanel.add(StartStopButton);

        ClearButton.setFont(new java.awt.Font("Comic Sans MS", 1, 14));
        ClearButton.setText("Clear");
        ClearButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                ClearButtonMouseClicked(evt);
            }
        });

        gB.setConstraints( ClearButton, gbC );
        buttonPanel.add(ClearButton);

        LogData.setFont(new java.awt.Font("Comic Sans MS", 1, 14));
        LogData.setText("Log data...");
        LogData.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                LogDataActionPerformed(evt);
            }
        });

        gB.setConstraints( LogData, gbC );
        buttonPanel.add(LogData);

        mute = new JButton( unmutedIcon );
        mute.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                MuteActionPerformed(evt);
            }
        });

        gbC.gridwidth = GridBagConstraints.REMAINDER;
        gB.setConstraints( mute, gbC );
        buttonPanel.add(mute);

        return buttonPanel;
}

    private void LogDataActionPerformed(java.awt.event.ActionEvent evt)
    {

        logDataFrame.setVisible(true);
    }

    private void MuteActionPerformed(java.awt.event.ActionEvent evt)
    {
        if ( isMuted )
	{
	    mute.setIcon(unmutedIcon);
            isMuted = false;
            audioClip.play();
        }
        else
	{
	    mute.setIcon(mutedIcon);
            isMuted = true;
        }
    }

    private void ClearButtonMouseClicked(java.awt.event.MouseEvent evt)
    {

        sendWriteOp(OpCtrClear,0,0x3f); //- (sendClears,X,AllCtrs)
        for (int i=0;i<Cosmixer.nCoincidences;i++)
	{
            counterPanel[i].clear();
	}
        
    }

    private void StartStopButtonMouseClicked(java.awt.event.MouseEvent evt)
    {
        int action;
        if(StartStopButton.isSelected()) {
            action = 1; // - Enable counters
            StartStopButton.setText("Stop"); // - Next press stops them again
        }
        else { // Need to stop counters
            action = 0; // - Stop counters
            StartStopButton.setText("Start"); // - Next Press starts them again
        }
        sendWriteOp(OpCtrEnable,action,0x3f); // - (sendEnables,set/unsetEnables,allCtrs)
    }

    private void exitForm(java.awt.event.WindowEvent evt)
    {
        usb.mjgCloseUSB();
        isUsbOpen = false;
        System.exit(0);
    }

// Send generic control packet
// op   - opcode
// arg  - opcode dependent argument
// data - opcode dependent data

    static private void sendWriteOp(int op, int arg, int data)
    {
        int outPacket [] = new int[68];
        int inPacket [] = new int[68];

        outPacket[0] = op|OpWriteExec;
        outPacket[1] = arg;
        outPacket[2] = 2;
        outPacket[3] = 0;

        outPacket[4] = data & 0xff;
        outPacket[5] = (data>>8) & 0xff;

        try
        {
          sendUSB(outPacket);
	}
        catch ( IOException ioe )
	{
	    usb.mjgCloseUSB();
            isUsbOpen = false;
	}

	return;

    }

    static private void getUSB(int[] buf) throws IOException
    {
        if ( !isUsbOpen ) return;
        if(usb.mjgRead(buf)!=0) throw new IOException();
    }

    static private void sendUSB(int[] buf) throws IOException
    {
        if ( !isUsbOpen ) return;
        if(usb.mjgWrite(buf)!=0) throw new IOException();
    }

    private static void launch()
    {
        usb = new JniCosmix();

        Cosmixer MainFrame = new Cosmixer() ;
        MainFrame.pack();
        MainFrame.setVisible(true);
                
    }

   public static void main(String args[])
   {
        //Schedule a job for the event-dispatching thread:
        //creating and showing this application's GUI.
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                launch();
            }
        });  
    }
 
// Variables declaration

    private DeviceControlPanel deviceControlPanel;
    private DataLoggerControl dataLoggerControl;
    private javax.swing.JFrame logDataFrame;
    private javax.swing.JButton ClearButton;
    private CounterPanel [] counterPanel;
    private javax.swing.JLabel MainLabel;
    private javax.swing.JButton LogData;
    private javax.swing.JButton mute;
    private javax.swing.JToggleButton StartStopButton;
    private ThresholdPanel [] thresholdPanel;
    private CoincidenceSetupPanel [] coPanels;
    private AdcCalibrationPanel adcCalibrationPanel;
    private JFrame adcCalibrationFrame;
    private AdcPanel adcPanel;
    private JFrame hvFrame;
    private HvPanel hvPanel;

    private URL audioURL;
    private AudioClip audioClip;
    private ImageIcon mutedIcon;
    private ImageIcon unmutedIcon;

// End of variables declaration


}

