
package mjg.instrument;

import java.awt.Color;
import java.awt.Font;
import java.text.SimpleDateFormat;

import javax.swing.JPanel;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.DateAxis;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.time.Minute;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.data.xy.XYDataset;
import org.jfree.data.Range;
import org.jfree.ui.ApplicationFrame;
import org.jfree.ui.RectangleInsets;
import org.jfree.ui.RefineryUtilities;

public class TimeSeriesChart
{
  public TimeSeries series;
  public ChartPanel chartPanel;

  public TimeSeriesChart(String title)
  {
    series = new TimeSeries("Rate", Minute.class);
    series.setMaximumItemCount( 50 );
    TimeSeriesCollection dataset = new TimeSeriesCollection();
    dataset.addSeries(series);

    JFreeChart chart = ChartFactory.createTimeSeriesChart(
            title,  // title
            null,             // x-axis label
            null,   // y-axis label
            dataset,            // data
            false,               // create legend?
            true,               // generate tooltips?
            false               // generate URLs?
        );

    chart.setBackgroundPaint(Color.white);

    XYPlot plot = (XYPlot) chart.getPlot();
    plot.setBackgroundPaint(Color.lightGray);
    plot.setDomainGridlinePaint(Color.white);
    plot.setRangeGridlinePaint(Color.white);
    plot.setAxisOffset(new RectangleInsets(5.0, 5.0, 5.0, 5.0));
    plot.setDomainCrosshairVisible(true);
    plot.setRangeCrosshairVisible(true);
        
    XYItemRenderer r = plot.getRenderer();
    if (r instanceof XYLineAndShapeRenderer)
    {
      XYLineAndShapeRenderer renderer = (XYLineAndShapeRenderer) r;
      renderer.setBaseShapesVisible(true);
      renderer.setBaseShapesFilled(true);
    }
        
    plot.getRangeAxis().setRange(new Range(0.0,20.0));
    plot.getRangeAxis().setTickLabelFont(new Font("SansSerif",Font.PLAIN,14));
    plot.getDomainAxis().setTickLabelFont(new Font("SansSerif",Font.PLAIN,14));

    chartPanel = new ChartPanel(chart);
    chartPanel.setPreferredSize(new java.awt.Dimension(700, 120));
    chartPanel.setMouseZoomable(true, false);

  }

  public TimeSeries getSeries(){ return series; }
  public ChartPanel getChartPanel(){ return chartPanel; }

}
